@@ -16,8 +16,9 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-
+#ifndef WIN32
+# include <unistd.h>
+#endif
 #include "lo/lo.h"
 
 int done = 0;
@@ -25,13 +26,19 @@ int done = 0;
 void error(int num, const char *m, const char *path);
 
 int generic_handler(const char *path, const char *types, lo_arg ** argv,
-                    int argc, void *data, void *user_data);
+                    int argc, lo_message data, void *user_data);
 
 int foo_handler(const char *path, const char *types, lo_arg ** argv,
-                int argc, void *data, void *user_data);
+                int argc, lo_message data, void *user_data);
+
+int blobtest_handler(const char *path, const char *types, lo_arg ** argv,
+                     int argc, lo_message data, void *user_data);
+
+int pattern_handler(const char *path, const char *types, lo_arg ** argv,
+                    int argc, lo_message data, void *user_data);
 
 int quit_handler(const char *path, const char *types, lo_arg ** argv,
-                 int argc, void *data, void *user_data);
+                 int argc, lo_message data, void *user_data);
 
 int main()
 {
@@ -45,6 +52,16 @@ int main()
      * to float and int */
     lo_server_thread_add_method(st, "/foo/bar", "fi", foo_handler, NULL);
 
+    /* add method that will match the path /blobtest with one blob arg */
+    lo_server_thread_add_method(st, "/blobtest", "b", blobtest_handler, NULL);
+
+    /* catch any message starting with /g using a pattern method */
+    lo_server_thread_add_method(st, "/p*", "", pattern_handler, NULL);
+
+    /* also catch "/q*", but glob_handle returns 1, so quit_handler
+     * gets called after */
+    lo_server_thread_add_method(st, "/q*", "", pattern_handler, NULL);
+
     /* add method that will match the path /quit with no args */
     lo_server_thread_add_method(st, "/quit", "", quit_handler, NULL);
 
@@ -72,11 +89,11 @@ void error(int num, const char *msg, const char *path)
 /* catch any incoming messages and display them. returning 1 means that the
  * message has not been fully handled and the server should try other methods */
 int generic_handler(const char *path, const char *types, lo_arg ** argv,
-                    int argc, void *data, void *user_data)
+                    int argc, lo_message data, void *user_data)
 {
     int i;
 
-    printf("path: <%s>\n", path);
+    printf("generic handler; path: <%s>\n", path);
     for (i = 0; i < argc; i++) {
         printf("arg %d '%c' ", i, types[i]);
         lo_arg_pp((lo_type)types[i], argv[i]);
@@ -89,17 +106,58 @@ int generic_handler(const char *path, const char *types, lo_arg ** argv,
 }
 
 int foo_handler(const char *path, const char *types, lo_arg ** argv,
-                int argc, void *data, void *user_data)
+                int argc, lo_message data, void *user_data)
 {
     /* example showing pulling the argument values out of the argv array */
-    printf("%s <- f:%f, i:%d\n\n", path, argv[0]->f, argv[1]->i);
+    printf("foo: %s <- f:%f, i:%d\n\n", path, argv[0]->f, argv[1]->i);
     fflush(stdout);
 
     return 0;
 }
 
+int blobtest_handler(const char *path, const char *types, lo_arg ** argv,
+                     int argc, lo_message data, void *user_data)
+{
+    /* example showing how to get data for a blob */
+    int i, size = argv[0]->blob.size;
+    char mydata[6];
+
+    unsigned char *blobdata = (unsigned char*)lo_blob_dataptr((lo_blob)argv[0]);
+    int blobsize = lo_blob_datasize((lo_blob)argv[0]);
+
+    /* Alternatively:
+     * blobdata = &argv[0]->blob.data;
+     * blobsize = argv[0]->blob.size;
+     */
+
+    /* Don't trust network input! Blob can be anything, so check if
+       each character is a letter A-Z. */
+    for (i=0; i<6 && i<blobsize; i++)
+        if (blobdata[i] >= 'A' && blobdata[i] <= 'Z')
+            mydata[i] = blobdata[i];
+        else
+            mydata[i] = '.';
+    mydata[5] = 0;
+
+    printf("%s <- length:%d '%s'\n", path, size, mydata);
+    fflush(stdout);
+
+    return 0;
+}
+
+int pattern_handler(const char *path, const char *types, lo_arg ** argv,
+                    int argc, lo_message data, void *user_data)
+{
+    printf("pattern handler matched: %s\n\n", path);
+    fflush(stdout);
+
+    // Let the dispatcher continue by returning non-zero, so
+    // quit_handler can also catch the message
+    return 1;
+}
+
 int quit_handler(const char *path, const char *types, lo_arg ** argv,
-                 int argc, void *data, void *user_data)
+                 int argc, lo_message data, void *user_data)
 {
     done = 1;
     printf("quiting\n\n");
