@@ -74,6 +74,7 @@
 #include "lo_types_internal.h"
 #include "lo_internal.h"
 #include "lo/lo.h"
+#include "lo/lo_lowlevel.h"
 #include "lo/lo_throw.h"
 
 typedef struct {
@@ -101,7 +102,8 @@ static lo_server lo_server_new_with_proto_internal(const char *group,
                                                    const char *iface,
                                                    const char *ip,
                                                    int proto,
-                                                   lo_err_handler err_h);
+                                                   lo_err_handler err_h,
+                                                   void *err_h_context);
 static int lo_server_join_multicast_group(lo_server s, const char *group,
                                           int family,
                                           const char *iface, const char *ip);
@@ -233,7 +235,7 @@ lo_server lo_server_new(const char *port, lo_err_handler err_h)
 lo_server lo_server_new_multicast(const char *group, const char *port,
                                   lo_err_handler err_h)
 {
-    return lo_server_new_with_proto_internal(group, port, 0, 0, LO_UDP, err_h);
+    return lo_server_new_with_proto_internal(group, port, 0, 0, LO_UDP, err_h, 0);
 }
 
 #if defined(WIN32) || defined(_MSC_VER) || defined(HAVE_GETIFADDRS)
@@ -241,14 +243,14 @@ lo_server lo_server_new_multicast_iface(const char *group, const char *port,
                                         const char *iface, const char *ip,
                                         lo_err_handler err_h)
 {
-    return lo_server_new_with_proto_internal(group, port, iface, ip, LO_UDP, err_h);
+    return lo_server_new_with_proto_internal(group, port, iface, ip, LO_UDP, err_h, 0);
 }
 #endif
 
 lo_server lo_server_new_with_proto(const char *port, int proto,
                                    lo_err_handler err_h)
 {
-    return lo_server_new_with_proto_internal(NULL, port, 0, 0, proto, err_h);
+    return lo_server_new_with_proto_internal(NULL, port, 0, 0, proto, err_h, 0);
 }
 
 lo_server lo_server_new_from_url(const char *url,
@@ -267,7 +269,7 @@ lo_server lo_server_new_from_url(const char *url,
         group = lo_url_get_hostname(url);
         port = lo_url_get_port(url);
         s = lo_server_new_with_proto_internal(group, port, 0, 0,
-                                              protocol, err_h);
+                                              protocol, err_h, 0);
         if (group)
             free(group);
         if (port)
@@ -276,7 +278,7 @@ lo_server lo_server_new_from_url(const char *url,
     } else if (protocol == LO_UNIX) {
         port = lo_url_get_path(url);
         s = lo_server_new_with_proto_internal(0, port, 0, 0,
-                                              LO_UNIX, err_h);
+                                              LO_UNIX, err_h, 0);
         if (port)
             free(port);
 #endif
@@ -294,6 +296,20 @@ lo_server lo_server_new_from_url(const char *url,
     return s;
 }
 
+lo_server lo_server_new_from_config(lo_server_config *config)
+{
+    if (config->size < sizeof(lo_server_config))
+        return NULL;
+
+    return lo_server_new_with_proto_internal(config->group,
+                                             config->port,
+                                             config->iface,
+                                             config->ip,
+                                             config->proto,
+                                             config->err_handler,
+                                             config->err_handler_context);
+}
+
 static
 void lo_server_resolve_hostname(lo_server s)
 {
@@ -399,7 +415,7 @@ static int lo_server_set_iface(lo_server s, int fam, const char *iface, const ch
 
     if (s->addr_if.size == sizeof(struct in_addr)) {
         if (setsockopt(s->sockets[0].fd, IPPROTO_IP, IP_MULTICAST_IF,
-                       (const char*)&s->addr_if.a.addr, s->addr_if.size) < 0)
+                       (const char*)&s->addr_if.a.addr, (socklen_t) s->addr_if.size) < 0)
 		{
             err = geterror();
             lo_throw(s, err, strerror(err), "setsockopt(IP_MULTICAST_IF)");
@@ -409,7 +425,7 @@ static int lo_server_set_iface(lo_server s, int fam, const char *iface, const ch
 #ifdef ENABLE_IPV6 // TODO: this setsockopt fails on linux
     else if (s->addr_if.size == sizeof(struct in6_addr)) {
         if (setsockopt(s->sockets[0].fd, IPPROTO_IP, IPV6_MULTICAST_IF,
-                       &s->addr_if.a.addr6, s->addr_if.size) < 0) {
+                       &s->addr_if.a.addr6, (socklen_t) s->addr_if.size) < 0) {
             err = geterror();
             lo_throw(s, err, strerror(err), "setsockopt(IPV6_MULTICAST_IF)");
             return err;
@@ -426,7 +442,8 @@ lo_server lo_server_new_with_proto_internal(const char *group,
                                             const char *iface,
                                             const char *ip,
                                             int proto,
-                                            lo_err_handler err_h)
+                                            lo_err_handler err_h,
+                                            void *err_h_context)
 {
     lo_server s;
     struct addrinfo *ai = NULL, *it, *used;
@@ -462,6 +479,7 @@ lo_server lo_server_new_with_proto_internal(const char *group,
         return 0;
 
     s->err_h = err_h;
+    s->error_user_data = err_h_context;
     s->first = NULL;
     s->ai = NULL;
     s->hostname = NULL;
@@ -663,7 +681,7 @@ lo_server lo_server_new_with_proto_internal(const char *group,
              0)) {
             err = geterror();
 #ifdef WIN32
-            if (err == EINVAL || err == WSAEADDRINUSE) {
+            if (err == WSAEINVAL || err == WSAEADDRINUSE) {
 #else
             if (err == EINVAL || err == EADDRINUSE) {
 #endif
@@ -905,8 +923,8 @@ void *lo_server_recv_raw(lo_server s, size_t * size)
 
     s->addr_len = sizeof(s->addr);
 
-    ret = recvfrom(s->sockets[0].fd, buffer, s->max_msg_size, 0,
-                   (struct sockaddr *) &s->addr, &s->addr_len);
+    ret = (int) recvfrom(s->sockets[0].fd, buffer, s->max_msg_size, 0,
+                         (struct sockaddr *) &s->addr, &s->addr_len);
     if (ret <= 0) {
         if (heap_buffer) free(buffer);
         return NULL;
@@ -1018,19 +1036,34 @@ static
 uint32_t lo_server_buffer_contains_msg(lo_server s, int isock)
 {
     struct socket_context *sc = &s->contexts[isock];
-    if (sc->buffer_read_offset > sizeof(uint32_t))
+    if (sc->buffer_read_offset <= sizeof(uint32_t))
+        return 0;
+
+    uint32_t msg_len = ntohl(*(uint32_t*)sc->buffer);
+    char *str = (char*)(sc->buffer + sizeof(uint32_t));
+    if (str[0] != '/' && str[0] != '#')
     {
-        uint32_t msg_len = ntohl(*(uint32_t*)sc->buffer);
-        return (msg_len + sizeof(uint32_t) <= sc->buffer_read_offset)
-            ? msg_len : 0;
+        // invalid message
+        goto clear_buffer;
     }
+    if (msg_len + sizeof(uint32_t) > sc->buffer_read_offset)
+    {
+        // still waiting for the rest of message
+        return 0;
+    }
+    if (msg_len && lo_validate_string(str, msg_len) < 0)
+        goto clear_buffer;
+    return msg_len;
+
+clear_buffer:
+    sc->buffer_read_offset = sc->buffer_msg_offset = 0;
     return 0;
 }
 
 static
 void *lo_server_buffer_copy_for_dispatch(lo_server s, int isock, size_t *psize)
 {
-	void *data;
+    void *data;
     struct socket_context *sc = &s->contexts[isock];
     uint32_t msg_len = lo_server_buffer_contains_msg(s, isock);
     if (msg_len == 0)
@@ -1063,17 +1096,18 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
     char *stack_buffer = 0, *read_into;
     uint32_t msg_len;
 	int buffer_bytes_left, bytes_recv;
-	ssize_t bytes_wrote, size;
+	ssize_t bytes_written, size;
     *pdata = 0;
 
   again:
 
     // Check if there is already a message waiting in the buffer.
-    if ((*pdata = lo_server_buffer_copy_for_dispatch(s, isock, psize)))
+    if ((*pdata = lo_server_buffer_copy_for_dispatch(s, isock, psize))) {
         // There could be more data, so return true.
         return 1;
+    }
 
-    buffer_bytes_left = sc->buffer_size - sc->buffer_read_offset;
+    buffer_bytes_left = (int) (sc->buffer_size - sc->buffer_read_offset);
 
     // If we need more than half the buffer, double the buffer size.
     size = sc->buffer_size;
@@ -1093,20 +1127,21 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
             break;
         }
 
-        buffer_bytes_left = size - sc->buffer_read_offset;
+        buffer_bytes_left = (int) (size - sc->buffer_read_offset);
     }
 
     if ((size_t)size > sc->buffer_size)
     {
         sc->buffer_size = size;
         sc->buffer = (char*) realloc(sc->buffer, sc->buffer_size);
-        if (!sc->buffer)
+        if (!sc->buffer) {
             // Out of memory
             return 0;
+        }
     }
 
     // Read as much as we can into the remaining buffer memory.
-    buffer_bytes_left = sc->buffer_size - sc->buffer_read_offset;
+    buffer_bytes_left = (int) (sc->buffer_size - sc->buffer_read_offset);
 
     read_into = sc->buffer + sc->buffer_read_offset;
 
@@ -1117,9 +1152,9 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
         read_into = stack_buffer;
     }
 
-    bytes_recv = recv(s->sockets[isock].fd,
-                      read_into,
-                      buffer_bytes_left, 0);
+    bytes_recv = (int) recv(s->sockets[isock].fd,
+                            read_into,
+                            buffer_bytes_left, 0);
 
     if (bytes_recv <= 0)
     {
@@ -1176,9 +1211,9 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
                            &sc->slip_state, &bytes_read) == 0)
         {
             // We have a whole message in the buffer.
-            size_t bytes_wrote = buffer_after - sc->buffer - sc->buffer_read_offset;
+            size_t bytes_written = buffer_after - sc->buffer - sc->buffer_read_offset;
 
-            sc->buffer_read_offset += bytes_wrote;
+            sc->buffer_read_offset += bytes_written;
 
             msg_len = sc->buffer_read_offset - sc->buffer_msg_offset - sizeof(uint32_t);
 
@@ -1211,8 +1246,8 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
 
         // Any data left over is left in the buffer, so update the
         // read offset to indicate the end of it.
-        bytes_wrote = buffer_after - sc->buffer - sc->buffer_read_offset;
-        sc->buffer_read_offset += bytes_wrote;
+        bytes_written = buffer_after - sc->buffer - sc->buffer_read_offset;
+        sc->buffer_read_offset += bytes_written;
     }
     else
     {
@@ -1274,8 +1309,7 @@ void *lo_server_recv_raw_stream(lo_server s, size_t * size, int *psock)
     poll(s->sockets, s->sockets_len, -1);
 
     for (i = s->sockets_len - 1; i >= 0 && !data; --i) {
-        if (s->sockets[i].revents == POLLERR
-            || s->sockets[i].revents == POLLHUP) {
+        if (s->sockets[i].revents & (POLLERR | POLLHUP)) {
             if (i > 0) {
                 closesocket(s->sockets[i].fd);
                 lo_server_del_socket(s, i, s->sockets[i].fd);
@@ -1297,9 +1331,10 @@ void *lo_server_recv_raw_stream(lo_server s, size_t * size, int *psock)
     FD_ZERO(&ps);
     for (i = (s->sockets_len - 1); i >= 0; --i) {
         FD_SET(s->sockets[i].fd, &ps);
+#ifndef WIN32
         if (s->sockets[i].fd > nfds)
             nfds = s->sockets[i].fd;
-
+#endif
         if ((data = lo_server_buffer_copy_for_dispatch(s, i, size))) {
             *psock = s->sockets[i].fd;
             return data;
@@ -1324,7 +1359,6 @@ void *lo_server_recv_raw_stream(lo_server s, size_t * size, int *psock)
                 sock = accept(sock, (struct sockaddr *) &addr, &addr_len);
 
                 i = lo_server_add_socket(s, sock, 0, &addr, addr_len);
-                init_context(&s->contexts[i]);
 
                 /* after adding a new socket, call select()/poll()
                  * again, since we are supposed to block until a
@@ -1360,7 +1394,7 @@ int lo_server_wait(lo_server s, int timeout)
 
 int lo_servers_wait(lo_server *s, int *status, int num_servers, int timeout)
 {
-    int i, j, sched_timeout;
+    int i, j, k, sched_timeout;
 
     if (!status)
         status = alloca(sizeof(int) * num_servers);
@@ -1384,19 +1418,23 @@ int lo_servers_wait(lo_server *s, int *status, int num_servers, int timeout)
 
   again:
     num_sockets = 0;
-    for (j = 0; j < num_servers; j++) {
+    for (j = 0, k = 0; j < num_servers; j++) {
         for (i = 0; i < s[j]->sockets_len; i++) {
             if (lo_server_buffer_contains_msg(s[j], i)) {
                 status[j] = 1;
+                ++k;
             }
             ++num_sockets;
         }
     }
 
+    // Return immediately if one or more servers already have messages waiting.
+    if (k > 0)
+      return k;
+
     struct pollfd *sockets = alloca(sizeof(struct pollfd) * num_sockets);
 
     sched_timeout = timeout;
-    int k;
     for (j = 0, k = 0; j < num_servers; j++) {
         for (i = 0; i < s[j]->sockets_len; i++) {
             sockets[k].fd = s[j]->sockets[i].fd;
@@ -1416,39 +1454,43 @@ int lo_servers_wait(lo_server *s, int *status, int num_servers, int timeout)
     // If poll() was reporting a new connection on the listening
     // socket rather than a ready message, accept it and check again.
     for (j = 0, k = 0; j < num_servers; j++) {
-        if (sockets[k].revents && sockets[k].revents != POLLERR
-            && sockets[k].revents != POLLHUP) {
-            if (s[j]->protocol == LO_TCP) {
-                int sock = accept(sockets[k].fd, (struct sockaddr *) &addr[j],
-                                  &addr_len);
+        if (sockets[k].revents) {
+            if (sockets[k].revents & (POLLIN | POLLPRI)) {
+                if (s[j]->protocol == LO_TCP) {
+                    int sock = accept(sockets[k].fd, (struct sockaddr *) &addr[j],
+                                      &addr_len);
 
-                i = lo_server_add_socket(s[j], sock, 0, &addr[j], addr_len);
-                if (i < 0)
-                    closesocket(sock);
+                    i = lo_server_add_socket(s[j], sock, 0, &addr[j], addr_len);
+                    if (i < 0)
+                        closesocket(sock);
 
-                init_context(&s[j]->contexts[i]);
+                    lo_timetag_now(&now);
 
-                lo_timetag_now(&now);
+                    double diff = lo_timetag_diff(now, then);
 
-                double diff = lo_timetag_diff(now, then);
+                    timeout -= (int)(diff*1000);
+                    if (timeout < 0)
+                        timeout = 0;
 
-                timeout -= (int)(diff*1000);
-                if (timeout < 0)
-                    timeout = 0;
-
-                goto again;
-            }
-            else {
-                status[j] = 1;
+                    goto again;
+                }
+                else {
+                    status[j] = 1;
+                }
             }
         }
         k += s[j]->sockets_len;
     }
 
-    for (j = 0, k = 1; j < num_servers; j++, k++) {
-        for (i = 1; i < s[j]->sockets_len; i++, k++) {
-            if (sockets[k].revents && sockets[k].revents != POLLERR
-                && sockets[k].revents != POLLHUP)
+    for (j = num_servers - 1, k = num_sockets - 1; j >= 0; j--, k--) {
+        for (i = s[j]->sockets_len - 1; i > 0; i--, k--) {
+            if (!sockets[k].revents)
+                continue;
+            if (sockets[k].revents & (POLLERR | POLLHUP)) {
+                closesocket(sockets[k].fd);
+                lo_server_del_socket(s[j], i, sockets[k].fd);
+            }
+            else
                 status[j] = 1;
         }
     }
@@ -1483,9 +1525,10 @@ int lo_servers_wait(lo_server *s, int *status, int num_servers, int timeout)
     for (j = 0; j < num_servers; j++) {
         for (i = 0; i < s[j]->sockets_len; i++) {
             FD_SET(s[j]->sockets[i].fd, &ps);
+#ifndef WIN32
             if (s[j]->sockets[i].fd > nfds)
                 nfds = s[j]->sockets[i].fd;
-
+#endif
             if (lo_server_buffer_contains_msg(s[j], i)) {
                 status[j] = 1;
             }
@@ -1512,8 +1555,6 @@ int lo_servers_wait(lo_server *s, int *status, int num_servers, int timeout)
                     if (i < 0)
                         closesocket(sock);
 
-                    init_context(&s[j]->contexts[i]);
-
                     lo_timetag_now(&now);
 
                     // Subtract time waited from total timeout
@@ -1623,12 +1664,17 @@ int lo_server_recv(lo_server s)
         poll(s->sockets, s->sockets_len, (int) (sched_time * 1000.0));
 
         for (i = 0; i < s->sockets_len; i++) {
-            if (s->sockets[i].revents == POLLERR
-                || s->sockets[i].revents == POLLHUP)
-                return 0;
-
-            if (s->sockets[i].revents)
-                break;
+            if (!s->sockets[i].revents)
+                continue;
+            if (s->sockets[i].revents & (POLLERR | POLLHUP)) {
+                if (i > 0) {
+                    closesocket(s->sockets[i].fd);
+                    lo_server_del_socket(s, i, s->sockets[i].fd);
+                    continue;
+                } else
+                    return 0;
+            }
+            break;
         }
 
         if (i >= s->sockets_len) {
@@ -1649,9 +1695,10 @@ int lo_server_recv(lo_server s)
         FD_ZERO(&ps);
         for (i = 0; i < s->sockets_len; i++) {
             FD_SET(s->sockets[i].fd, &ps);
+#ifndef WIN32
             if (s->sockets[i].fd > nfds)
                 nfds = s->sockets[i].fd;
-
+#endif
             if (s->protocol == LO_TCP
                 && (data = lo_server_buffer_copy_for_dispatch(s, i, &size)))
             {
@@ -1695,7 +1742,7 @@ int lo_server_recv(lo_server s)
         return -1;
     }
     free(data);
-    return size;
+    return (int) size;
 }
 
 int lo_server_add_socket(lo_server s, int socket, lo_address a,
@@ -1728,13 +1775,14 @@ int lo_server_add_socket(lo_server s, int socket, lo_address a,
         if (!sc)
             return -1;
         s->contexts = (struct socket_context *) sc;
-        memset((char*)sc + s->sockets_alloc*sizeof(*s->contexts),
-               0, s->sockets_alloc*sizeof(*s->contexts));
 
         s->sockets_alloc *= 2;
     }
 
+    /* Initialize new socket and context */
     s->sockets[s->sockets_len].fd = socket;
+    init_context(&s->contexts[s->sockets_len]);
+
     s->sockets_len++;
 
     /* Update socket-indexed array of sources */
@@ -1774,8 +1822,10 @@ void lo_server_del_socket(lo_server s, int index, int socket)
     lo_address_free_mem(&s->sources[s->sockets[index].fd]);
     cleanup_context(&s->contexts[index]);
 
-    for (i = index + 1; i < s->sockets_len; i++)
+    for (i = index + 1; i < s->sockets_len; i++) {
         s->sockets[i - 1] = s->sockets[i];
+        s->contexts[i - 1] = s->contexts[i];
+    }
     s->sockets_len--;
 }
 
@@ -1786,8 +1836,8 @@ static int dispatch_data(lo_server s, void *data,
     char *path = (char*) data;
     ssize_t len = lo_validate_string(data, size);
     if (len < 0) {
-        lo_throw(s, -len, "Invalid message path", NULL);
-        return len;
+        lo_throw(s, (int) -len, "Invalid message path", NULL);
+        return (int) len;
     }
 
     if (!strcmp((const char *) data, "#bundle")) {
@@ -1798,11 +1848,11 @@ static int dispatch_data(lo_server s, void *data,
 
         ssize_t bundle_result = lo_validate_bundle(data, size);
         if (bundle_result < 0) {
-            lo_throw(s, -bundle_result, "Invalid bundle", NULL);
-            return bundle_result;
+            lo_throw(s, (int) -bundle_result, "Invalid bundle", NULL);
+            return (int) bundle_result;
         }
         pos = (char*) data + len;
-        remain = size - len;
+        remain = (int) (size - len);
 
         lo_timetag_now(&now);
         ts.sec = lo_otoh32(*((uint32_t *) pos));
@@ -1866,7 +1916,7 @@ static int dispatch_data(lo_server s, void *data,
         dispatch_method(s, (const char *)data, msg, sock);
         lo_message_free(msg);
     }
-    return size;
+    return (int) size;
 }
 
 int lo_server_dispatch_data(lo_server s, void *data, size_t size)
@@ -1897,10 +1947,10 @@ static void dispatch_method(lo_server s, const char *path,
                             lo_message msg, int sock)
 {
     char *types = msg->types + 1;
-    int argc = strlen(types);
+    int argc = (int) strlen(types);
     lo_method it;
     int ret = 1;
-    int pattern = strpbrk(path, " #*,?[]{}") != NULL;
+    int pattern = lo_string_contains_pattern(path);
     lo_address src = 0;
     const char *pptr;
 
@@ -1930,14 +1980,15 @@ static void dispatch_method(lo_server s, const char *path,
     for (it = s->first; it; it = it->next) {
         /* If paths match or handler is wildcard */
         if (!it->path || !strcmp(path, it->path) ||
-            (pattern && lo_pattern_match(it->path, path))) {
+            (pattern && lo_pattern_match(it->path, path)) ||
+            (it->has_pattern && lo_pattern_match(path, it->path))) {
             /* If types match or handler is wildcard */
             if (!it->typespec || !strcmp(types, it->typespec)) {
                 /* Send wildcard path to generic handler, expanded path
                    to others.
                  */
                 pptr = path;
-                if (it->path)
+                if (it->path && !it->has_pattern)
                     pptr = it->path;
                 ret = it->handler(pptr, types, msg->argv, argc, msg,
                                   it->user_data);
@@ -1996,7 +2047,7 @@ static void dispatch_method(lo_server s, const char *path,
         /* if its a method enumeration call */
         if (pos && *(pos + 1) == '\0') {
             lo_message reply = lo_message_new();
-            int len = strlen(path);
+            int len = (int) strlen(path);
             lo_strlist *sl = NULL, *slit, *slnew, *slend;
 
             lo_arg **argv = msg->argv;
@@ -2011,13 +2062,11 @@ static void dispatch_method(lo_server s, const char *path,
                     char *tmp;
                     char *sec;
 
+                    int tmplen = (int) strlen(it->path + len) + 1;
                     tmp = (char*) malloc(strlen(it->path + len) + 1);
-                    strcpy(tmp, it->path + len);
-#if defined(WIN32) || defined(_MSC_VER)
+                    strncpy(tmp, it->path + len, tmplen);
+                    tmp[tmplen-1]=0;
                     sec = strchr(tmp, '/');
-#else
-                    sec = index(tmp, '/');
-#endif
                     if (sec)
                         *sec = '\0';
                     slend = sl;
@@ -2144,12 +2193,7 @@ lo_method lo_server_add_method(lo_server s, const char *path,
     lo_method m = (lo_method) calloc(1, sizeof(struct _lo_method));
     lo_method it;
 
-    if (path && strpbrk(path, " #*,?[]{}")) {
-        if (m) {
-            free(m);
-        }
-        return NULL;
-    }
+    m->has_pattern = lo_string_contains_pattern(path);
 
     if (path) {
         m->path = strdup(path);
@@ -2188,7 +2232,7 @@ void lo_server_del_method(lo_server s, const char *path,
     if (!s->first)
         return;
     if (path)
-        pattern = strpbrk(path, " #*,?[]{}") != NULL;
+        pattern = lo_string_contains_pattern(path);
 
     it = s->first;
     prev = it;
@@ -2244,11 +2288,9 @@ int lo_server_del_lo_method(lo_server s, lo_method m)
             } else {
                 prev->next = it->next;
             }
-            next = it->next;
             free((void *) it->path);
             free((void *) it->typespec);
             free(it);
-            it = prev;
             return 0;
         }
         prev = it;
