@@ -28,7 +28,13 @@
 #include <errno.h>
 #include <config.h>
 #include <limits.h>
-#include <unistd.h>
+#ifdef _MSC_VER
+  #if (__STDC_VERSION__ < 201112L)
+    #define strtok_r strtok_s
+  #endif
+#else
+  #include <unistd.h>
+#endif
 #include <lo/lo.h>
 
 static FILE* input_file = 0;
@@ -64,7 +70,7 @@ lo_message create_message(char **argv)
     int i, argi;
     lo_message message;
     const char *types;
-    char *arg;
+    char *arg = NULL;
     int values;
 
     message = lo_message_new();
@@ -267,7 +273,7 @@ double timetag_double(const lo_timetag tt)
 void timetag_multiply(lo_timetag *tt, double d)
 {
     d *= timetag_double(*tt);
-    tt->sec = floor(d);
+    tt->sec = (uint32_t) d;
     d -= tt->sec;
     tt->frac = (uint32_t) (d * (double)(1LL<<32));
 }
@@ -277,9 +283,9 @@ int send_file(lo_address target, double speed) {
     const char *delim = " \r\n";
     char *args[64];
     char str[1024], *p, *s, *path;
-    lo_timetag tt_start = {0, 0}, tt_start_file, tt_now;
+    lo_timetag tt_start = {0, 0}, tt_start_file = {0, 0}, tt_now;
     lo_timetag tt1 = {0, 0}, tt2 = {0, 0}, *tt_last = &tt1, *tt_this = &tt2;
-    int tt_init = 0, ret;
+    int tt_init = 0, ret = 0;
     lo_message m;
     lo_bundle b = 0;
 
@@ -354,7 +360,11 @@ int send_file(lo_address target, double speed) {
             lo_timetag_now(&tt_now);
             double wait_time = timetag_diff(*tt_last, tt_now);
             if (wait_time > 0.) {
-                usleep(wait_time * 1000000);
+#if defined(WIN32) || defined(_MSC_VER)
+                Sleep((DWORD)(wait_time * 1000));
+#else
+                usleep((useconds_t)(wait_time * 1000000));
+#endif
             }
             if (b) {
                 ret = lo_send_bundle(target, b);
@@ -376,7 +386,11 @@ int send_file(lo_address target, double speed) {
         lo_timetag_now(&tt_now);
         double wait_time = timetag_diff(*tt_last, tt_now);
         if (wait_time > 0.) {
-            usleep(wait_time * 1000000);
+#if defined(WIN32) || defined(_MSC_VER)
+            Sleep((DWORD)(wait_time * 1000));
+#else
+            usleep((useconds_t)(wait_time * 1000000));
+#endif
         }
         lo_send_bundle(target, b);
     }
